/****************************************************************************
**
** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
** Contact: Qt Software Information (qt-info@nokia.com)
**
** This file is part of the QtXMLPatterns module of the Qt Toolkit.
**
** No Commercial Usage
** This file contains pre-release code and may not be distributed.
** You may use this file in accordance with the terms and conditions
** contained in the either Technology Preview License Agreement or the
** Beta Release License Agreement.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the file LICENSE.GPL included in
** the packaging of this file.  Please review the following information
** to ensure GNU General Public Licensing requirements will be met:
** http://www.fsf.org/licensing/licenses/info/GPLv2.html and
** http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
** exception, Nokia gives you certain additional rights. These rights
** are described in the Nokia Qt GPL Exception version 1.3, included in
** the file GPL_EXCEPTION.txt in this package.
**
** Qt for Windows(R) Licensees
** As a special exception, Nokia, as the sole copyright holder for Qt
** Designer, grants users of the Qt/Eclipse Integration plug-in the
** right for the Qt/Eclipse Integration to link to functionality
** provided by Qt Designer and its related libraries.
**
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
**
****************************************************************************/

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.

#ifndef Patternist_ApplyTemplate_H
#define Patternist_ApplyTemplate_H

#include "qtemplatemode_p.h"

QT_BEGIN_HEADER
QT_BEGIN_NAMESPACE

namespace QPatternist
{
    /**
     * @short When combined with other components, implements
     * @c xsl:apply-templates.
     *
     * Note that ApplyTemplate isn't named ApplyTemplates. The reason for this
     * is that ApplyTemplate doesn't do the iteration part. An @c
     * <xsl:apply-templates/> instruction is rewritten into:
     *
     * @code
     * child::node/() map apply-template()
     * @endcode
     *
     * Hence, this expression requires a focus, although it can consist of
     * atomic values.
     *
     * @since 4.5
     * @author Frans Englich <fenglich@trolltech.com>
     * @ingroup Patternist_expressions
     */
    class ApplyTemplate : public TemplateInvoker
    {
    public:
        typedef QExplicitlySharedDataPointer<ApplyTemplate> Ptr;

        /**
         * @short @p mode may be @c null. If it is, ApplyTemplate interprets
         * that as that it should use the #current mode.
         *
         * @see StaticContext::currentTemplateMode()
         */
        ApplyTemplate(const TemplateMode::Ptr &mode,
                      const WithParam::Hash &withParams,
                      const TemplateMode::Ptr &defaultMode);

        virtual Item::Iterator::Ptr evaluateSequence(const DynamicContext::Ptr &context) const;

        virtual SequenceType::Ptr staticType() const;
        virtual ExpressionVisitorResult::Ptr accept(const ExpressionVisitor::Ptr &visitor) const;
        virtual Properties properties() const;

        /**
         * The only reason this function exists, is in order to convert
         * QXmlNodeModelIndex to QPatternist::Item. So, this is a huge
         * performance setback. It applies for one of the builtin templates.
         */
        inline Item mapToItem(const QXmlNodeModelIndex &node,
                              const DynamicContext::Ptr &context) const;
        inline Item::Iterator::Ptr mapToSequence(const Item &item,
                                                 const DynamicContext::Ptr &context) const;

        inline TemplateMode::Ptr mode() const;

        virtual bool configureRecursion(const CallTargetDescription::Ptr &sign);
        virtual Expression::Ptr body() const;
        virtual CallTargetDescription::Ptr callTargetDescription() const;

        Expression::Ptr compress(const StaticContext::Ptr &context);

    private:
        typedef QExplicitlySharedDataPointer<const ApplyTemplate> ConstPtr;

        Template::Ptr findTemplate(const DynamicContext::Ptr &context,
                                   const TemplateMode::Ptr &templateMode) const;
        /**
         * @note You typically want to use effectiveMode().
         */
        const TemplateMode::Ptr m_mode;

        TemplateMode::Ptr m_defaultMode;

        inline TemplateMode::Ptr effectiveMode(const DynamicContext::Ptr &context) const;
    };

    TemplateMode::Ptr ApplyTemplate::mode() const
    {
        return m_mode;
    }
}

QT_END_NAMESPACE
QT_END_HEADER

#endif
