/****************************************************************************
**
** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
** Contact: Qt Software Information (qt-info@nokia.com)
**
** This file is part of the QtXMLPatterns module of the Qt Toolkit.
**
** No Commercial Usage
** This file contains pre-release code and may not be distributed.
** You may use this file in accordance with the terms and conditions
** contained in the either Technology Preview License Agreement or the
** Beta Release License Agreement.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the file LICENSE.GPL included in
** the packaging of this file.  Please review the following information
** to ensure GNU General Public Licensing requirements will be met:
** http://www.fsf.org/licensing/licenses/info/GPLv2.html and
** http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
** exception, Nokia gives you certain additional rights. These rights
** are described in the Nokia Qt GPL Exception version 1.3, included in
** the file GPL_EXCEPTION.txt in this package.
**
** Qt for Windows(R) Licensees
** As a special exception, Nokia, as the sole copyright holder for Qt
** Designer, grants users of the Qt/Eclipse Integration plug-in the
** right for the Qt/Eclipse Integration to link to functionality
** provided by Qt Designer and its related libraries.
**
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
**
****************************************************************************/

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.

#ifndef Patternist_EBVExtractor_H
#define Patternist_EBVExtractor_H

#include "qsinglecontainer_p.h"

QT_BEGIN_HEADER

QT_BEGIN_NAMESPACE

namespace QPatternist
{
    /**
     * @short Calculates the Effective Boolean Value of its operand.
     *
     * EBVExtractor performs functionality wise the same as @c fn:boolean(),
     * but does it without the dependencies which FunctionCall requires.
     *
     * There is code-duplication going on with BooleanFN.
     *
     * @see BooleanFN
     * @author Frans Englich <fenglich@trolltech.com>
     * @ingroup Patternist_expressions
     */
    class EBVExtractor : public SingleContainer
    {
    public:
        EBVExtractor(const Expression::Ptr &operand);
        virtual bool evaluateEBV(const DynamicContext::Ptr &context) const;
        virtual SequenceType::List expectedOperandTypes() const;
        virtual const SourceLocationReflection *actualReflection() const;
        virtual Expression::Ptr typeCheck(const StaticContext::Ptr &context,
                                          const SequenceType::Ptr &reqType);
        virtual ExpressionVisitorResult::Ptr accept(const ExpressionVisitor::Ptr &visitor) const;

        /**
         * @short Returns always @c xs:boolean.
         */
        virtual SequenceType::Ptr staticType() const;

        template<typename TSubClass, typename ThisType>
        static Expression::Ptr typeCheck(const StaticContext::Ptr &context,
                                         const SequenceType::Ptr &reqType,
                                         ThisType *const caller)
        {
            if(*CommonSequenceTypes::EBV->itemType() == *reqType->itemType())
                return caller->operands().first()->typeCheck(context, reqType);
            else
                return caller->TSubClass::typeCheck(context, reqType);
        }

    };
}

QT_END_NAMESPACE

QT_END_HEADER

#endif
